	;**********************************************************************
	;                            OVEN.ASM                                 *
	;**********************************************************************
	;    Filename:	    oven.asm                                          *
	;    Date:          18/07/2010                                        *
	;    File Version:  1.0                                               *
	;    Author:        Sergio A. S. de Aguiar                            *
	;    Company:       www.ssaguiar.com - ssaguiar@ssaguiar.com          *
	;                                                                     *
	;**********************************************************************
	;    Files Required: P16F677.INC                                      *
	;**********************************************************************
	;    PIC 16F677 pinout:                                               *
	;                            |--\/--|                                 *
	;                       Vdd -|1   20|- Vss                            *
	;      RA5/T1CKI/OSC1/CLKIN -|2   19|- RA0/AN0/C1IN+/ICSPDAT/ULPWU    *
	;   RA4/AN3/T1G/OSC2/CLKOUT -|3   18|- RA1/AN1/C12IN0-/VREF/ICSPCLK   *
	;              RA3/MCLR/VPP -|4   17|- RA2/AN2/T0CKI/INT/C1OUT        *
	;                       RC5 -|5   16|- RC0/AN4/C2IN+                  *
	;                 RC4/C2OUT -|6   15|- RC1/AN5/C12IN1-                *
	;             RC3/AN7C12IN3 -|7   14|- RC2/AN6/C12IN2-                *
	;                RC6/AN8/SS -|8   13|- RB4/AN10/SDI/SDA               *
	;               RC7/AN9/SDO -|9   12|- RB5/AN11                       *
	;                       RB7 -|10  11|- RB6/SCK/SCL                    *
	;                            --------                                 *
	;                                                                     *
	;/*********************************************************************
	;*                             _\|/_                                  *
	;*                             (o o)                                  *
	;*_________________________ooOO_(_)_OOoo______________________________*
	;**********************************************************************
	;****        _        _    _         _    _    _    _    _         ****
	;****       | |   |   _|   _|  |_|  |_   |_     |  |_|  |_|        ****
	;****       |_|   |  |_    _|    |   _|  |_|    |  |_|   _|        ****
	;****                                                              ****
	;**********************************************************************/

;*****************************************************************************

;/****************************************************************************
;* DESCRIPTION: System definitions.
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

	ERRORLEVEL -302 ;remove message about using proper bank

	LIST  P=16F677
	INCLUDE <P16F677.INC>
	__CONFIG _BOR_OFF & _FCMEN_OFF & _IESO_OFF & _CPD_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTOSCIO

;/****************************************************************************
;* DESCRIPTION: Program variables and constant definitions.
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

TEMPER		EQU 0x20						; Store temperature
UNITS		EQU 0x21						; UNITS
TENS		EQU	0x22						; TENS
HUNDREDS	EQU 0x23						; HUNDREDS
AUX			EQU	0x24						; GENERAL aux. REGISTER
TIMER		EQU	0x25						; Timer register
;LEDS_S		EQU	0x26						; Leds state
DISPTMP		EQU	0x27
TMR_1MS		EQU	0x28
TMR_100MS	EQU	0x29
T1_CONTA	EQU	0x2A
INDEX_DISP	EQU	0x2B
STATUS_TEMP	EQU 0x2C						; REGISTRADOR DE STATUS TEMPORÁRIO
W_TEMP		EQU 0x2D						; REGISTRADOR DE TRABALHO TEMPORÁRIO
FSR_TEMP	EQU 0x2E						; REG. DE ENDERECO INDIRETO TEMPORÁRIO
PCLATH_TEMP	EQU 0x2F						; REGISTRADOR DE PAGINAÇÃO TEMPORÁRIO
BEEPCNT		EQU	0x30
DCNT0		EQU	0x31
DCNT1		EQU	0x32
BUZ_LENGTH	EQU 0x33						; Buzzer length
delay_temp	EQU	0x34						; Counter for ms delay routine
MODES		EQU	0x35
FLAGS		EQU	0x36						; General flags
KEYS_FLAGS	EQU	0x37						; Keys pressed flags
DELAY_TEMP	EQU	0x38
KEYUPCNT	EQU	0x39						; Key UP counter
KEYDWNCNT	EQU 0x3A						; Key DOWN counter
KEYONOFFCNT	EQU	0x3B						; Key ONOFF counter
KEY_DEBOUNCE	EQU	0x3C						; Key debounce counter
KEY_TURBO	EQU	0x3D						; Key turbo enable counter

#DEFINE		MODEPRE				MODES,0		; Pre heating mode
#DEFINE		MODETEMP			MODES,1		; Temperature mode
#DEFINE		MODETIMER			MODES,2		; Timer mode
#DEFINE		MODELIGHT			MODES,3		; Internal Light ON?
#DEFINE		MODETOASTER			MODES,4		; Toaster ON?
#DEFINE		LEDBLINK			MODES,5		; Is led ON/OFF Blinking?
#DEFINE		MODEPAUSE			MODES,6		; Oen in pause mode?
#DEFINE		OVENON				MODES,7		; Oven ON?

#DEFINE		BUZZEREN			FLAGS,0		; Buzzer pin ON/OFF Flag
#DEFINE		BUZZERCTL			FLAGS,1		; Buzzer ON?

; Leds
#DEFINE		LED_CMN				PORTC,2		; Leds Anodes

#DEFINE		LED_UP				PORTA,4		; Led UP
#DEFINE		LED_TIMER			PORTB,6		; Led TIMER
#DEFINE		LED_TEMP			PORTB,5		; Led TEMP
#DEFINE		LED_DWN				PORTA,2		; Led DWN
#DEFINE		LED_LIGHT			PORTB,4		; Led LIGHT
#DEFINE		LED_TOASTER			PORTA,5		; Led TOASTER
#DEFINE		LED_ONOFF			PORTC,6		; Led ONOFF
#DEFINE		TRIS_LED_ONOFF		TRISC,6		; Led ONOFF TRIS

; Displays
#DEFINE		DISP1				PORTC,3		; Anode display DIS0
#DEFINE		DISP2				PORTC,4		; Anode display DIS1
#DEFINE		DISP3				PORTC,5		; Anode display DIS2

; Keys
#DEFINE		KEY_UP				PORTB,7		; KEY_UP
#DEFINE		KEY_TIMER			PORTB,6		; KEY_TIMER
#DEFINE		KEY_ONOFF			PORTB,5		; KEY_ONOFF
#DEFINE		KEY_LIGHT			PORTB,4		; KEY_LIGHT
#DEFINE		KEY_TOASTER			PORTA,5		; KEY_TOASTER
#DEFINE		KEY_TEMP			PORTA,4		; KEY_TEMP
#DEFINE		KEY_DWN				PORTA,2		; KEY_DWN

; Buzzer
#DEFINE		BUZZER				PORTC,0		; Buzzer
#DEFINE		TMR0VAL				D'1'		; Timer0 preset value (1 =~ 100ms)

;/****************************************************************************
;* DESCRIPTION: Start of program.
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;***************************************************************************** 
	ORG		0x0000
	GOTO	SET_UP

;****************************************************************************
;* DESCRIPTION: Interrupts service (Timer0, Timer1,...).
;*
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************
	ORG		0x0004
	GOTO	INTERRUPTS_ISR

;/****************************************************************************
;* DESCRIPTION: Decodes value in W register to display segments for PORTA.
;* return:      result in W
;* ALGORITHM:   none
;* NOTES:
;
;*****************************************************************************/ 
DECDISP_A
	ADDWF   PCL,F		; compute the jump value
						;	|7|6|5|4|3|2|1|0| -	Bits PORTA
						;	|-|-|B|E|-|A|-|-|
	RETLW	B'00000000'	;	|-|-|B|E|-|A|-|-| -	0       _A_      
	RETLW	B'00010100'	;	|-|-|B|X|-|X|-|-| -	1     F|   |B    
	RETLW	B'00000000'	;	|-|-|B|E|-|A|-|-| -	2      |_G_|     
	RETLW	B'00010000'	;	|-|-|B|X|-|A|-|-| -	3     E|   |C    
	RETLW	B'00010100'	;	|-|-|B|X|-|X|-|-| -	4      |_D_|     
	RETLW	B'00110000'	;	|-|-|X|X|-|A|-|-| -	5                    
	RETLW	B'00100000'	;	|-|-|X|E|-|A|-|-| -	6
	RETLW	B'00010000'	;	|-|-|B|X|-|A|-|-| -	7
	RETLW	B'00000000'	;	|-|-|B|E|-|A|-|-| -	8
	RETLW	B'00010000'	;	|-|-|B|X|-|A|-|-| -	9
	RETLW	B'11001011'	;	|-|-|B|E|-|A|-|-| -	P
	RETLW	B'11101111'	;	|-|-|-|E|-|-|-|-| -	r
	RETLW	B'11101011'	;	|-|-|-|E|-|A|-|-| -	E
	RETLW	B'11101011'	;	|-|-|-|E|-|A|-|-| -	E
	RETLW	B'11101111'	;	|-|-|-|E|-|-|-|-| -	r
	RETLW	B'11101111'	;	|-|-|-|E|-|-|-|-| -	r

;/****************************************************************************
;* DESCRIPTION: Decodes value in W register to display segments for PORTB.
;* return:      result in W
;* ALGORITHM:   none
;* NOTES:
;*****************************************************************************/ 
DECDISP_B
	ADDWF   PCL,F		; compute the jump value
						;	|7|6|5|4|3|2|1|0| -	Bits PORTB
						;	|C|F|D|G|-|-|-|-| -
	RETLW	B'00010000'	;	|C|F|D|X|-|-|-|-| -	0       _A_    
	RETLW	B'01110000'	;	|C|X|X|X|-|-|-|-| -	1     F|   |B  
	RETLW	B'11000000'	;	|X|X|D|G|-|-|-|-| -	2      |_G_|   
	RETLW	B'01000000'	;	|C|X|D|G|-|-|-|-| -	3     E|   |C  
	RETLW	B'00100000'	;	|C|F|X|G|-|-|-|-| -	4      |_D_|   
	RETLW	B'00000000'	;	|C|F|D|G|-|-|-|-| -	5
	RETLW	B'00000000'	;	|C|F|D|G|-|-|-|-| -	6
	RETLW	B'01110000'	;	|C|X|X|X|-|-|-|-| -	7
	RETLW	B'00000000'	;	|C|F|D|G|-|-|-|-| -	8
	RETLW	B'00000000'	;	|C|F|D|G|-|-|-|-| -	9
	RETLW	B'10101111'	;	|-|F|-|G|-|-|-|-| -	P
	RETLW	B'11101111'	;	|-|-|-|G|-|-|-|-| -	r
	RETLW	B'10001111'	;	|-|F|D|G|-|-|-|-| -	E
	RETLW	B'10001111'	;	|-|F|D|G|-|-|-|-| -	E
	RETLW	B'11101111'	;	|-|-|-|G|-|-|-|-| -	r
	RETLW	B'11101111'	;	|-|-|-|G|-|-|-|-| -	r

;/****************************************************************************
;* DESCRIPTION: Interrupts ISR
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 
INTERRUPTS_ISR
	;/****************************************************************************
	;* DESCRIPTION: Save all context values, so we can process interrupts
	;/****************************************************************************
	MOVWF   W_TEMP			; save off current W register contents
	MOVF	STATUS,w		; move status register into W register
	MOVWF	STATUS_TEMP		; save off contents of STATUS register
	MOVF	FSR,W			; Save FSR register
	MOVWF	FSR_TEMP		; to FSR_TEMP
	MOVF	PCLATH,w		; move pclath register into w register
	MOVWF	PCLATH_TEMP		; save off contents of PCLATH register
	CLRF	PCLATH			; Clear PCLATH
							; (Select PAGE 0)
	CLRF	STATUS			; Clear STATUS
							; (Select BANK 0)

	;/****************************************************************************
	;* DESCRIPTION: Test to see if it was a Timer0 or Timer1 interrupt
	;/****************************************************************************
	BTFSC	PIE1,TMR1IE		; Timer1 overflow interrupt?
	GOTO	INT_TMR1		; Yes.
	BTFSS	INTCON,T0IF		; Timer0 overflow interrupt?
	GOTO	END_INT			; No (should never get here).

;****************************************************************************
;* For TIMER0 Interrupt calculation:
;* F(interrupt) = (256 - InitTMR0) * Prescaler
;* T(interrupt) = 1/(F(interrupt)) = 1/10 = 0.100s = 100ms
;* If you have a 4MHz crystal then your pic will run at 1MHz.
;* For 100Hz, set the prescaler to 128 and preload timer0 with 6 
;* (256-1 = 255, 255*256 = 200. Timer0 interrupt flag (T0IF) 
;* will become set after 50mS.
;*
;*****************************************************************************

INT_TMR0
	;/****************************************************************************
	;* DESCRIPTION: Clear Timer0 interrupt and reload it's values
	;/****************************************************************************
	BCF		INTCON,T0IF		; Clear Timer0 interrupt flag
	MOVLW	TMR0VAL			; Interrupt Timer 0 every 100ms (at 4MHz)
	MOVWF	TMR0			; Set TMR0 file

;/****************************************************************************
;* DESCRIPTION: Enable Port A and Port B pull-ups, so we can read keys
;/****************************************************************************
	MOVLW	B'11110000'		; Port B pins 7,6,5 and 4 pul-ups
	BCF		STATUS, RP0		; Goto
	BSF		STATUS, RP1		; BANK 2
	MOVWF	WPUB			; Enable pull-up resistors on PORTB
	BSF		STATUS, RP0		; Goto
	BCF		STATUS, RP1		; BANK 1
	MOVLW	B'00110100'		; Port A pins 5,4 and 2 pull-ups
	MOVWF	WPUA			; Enable pull-up resistors on PORTA
	BCF		OPTION_REG, NOT_RABPU ; Enable global pullups

;/****************************************************************************
;* DESCRIPTION: Verify if key Up is pressed
;/****************************************************************************
	BSF		TRISB,7			; Set PORTB,7 as input (KEY_UP)
	BCF		STATUS, RP0		; Goto BANK 0
	BTFSS	KEY_UP			; Test if PORTB pin 7 is 0 (KEY_UP pressed)
	GOTO	KEYUPISR		; If key pressed, go process it

;/****************************************************************************
;* DESCRIPTION: Verify if key Timer is pressed
;/****************************************************************************
	BCF		STATUS, RP1		; BANK 1
	BSF		TRISB,6			; Set PORTB,6 as input (KEY_TIMER)
	BCF		STATUS, RP0		; Goto BANK 0
	BTFSS	KEY_TIMER		; Test if KEY_TIMER pressed
	GOTO	KEYTIMERISR		; If key pressed, go process it

;/****************************************************************************
;* DESCRIPTION: Verify if key On/Off is pressed
;/****************************************************************************
	BCF		STATUS, RP1		; BANK 1
	BSF		TRISB,5			; Set PORTB,5 as input (KEY_ONOFF)
	BCF		STATUS, RP0		; Goto BANK 0
	BTFSS	KEY_ONOFF		; Test if KEY_ONOFF pressed
	GOTO	KEYONOFFISR		; If key pressed, go process it

;/****************************************************************************
;* DESCRIPTION: Verify if key Light is pressed
;/****************************************************************************
	BCF		STATUS, RP1		; BANK 1
	BSF		TRISB,4			; Set PORTB,4 as input (KEY_LIGHT)
	BCF		STATUS, RP0		; Goto BANK 0
	BTFSS	KEY_LIGHT		; Test if KEY_LIGHT pressed
	GOTO	KEYLIGHTISR		; If key pressed, go process it

;/****************************************************************************
;* DESCRIPTION: Verify if key Toaster is pressed
;/****************************************************************************
	BCF		STATUS, RP1		; BANK 1
	BSF		TRISA,5			; Set PORTA,5 as input (KEY_TOASTER)
	BCF		STATUS, RP0		; Goto BANK 0
	BTFSS	KEY_TOASTER		; Test if KEY_TOASTER pressed
	GOTO	KEYTOASTERISR	; If key pressed, go process it

;/****************************************************************************
;* DESCRIPTION: Verify if key Temperature is pressed
;/****************************************************************************
	BCF		STATUS, RP1		; BANK 1
	BSF		TRISA,4			; Set PORTA,4 as input (KEY_TEMP)
	BCF		STATUS, RP0		; Goto BANK 0
	BTFSS	KEY_TEMP		; Test if KEY_TEMP pressed
	GOTO	KEYTEMPISR		; If key pressed, go process it

;/****************************************************************************
;* DESCRIPTION: Verify if key Down is pressed
;/****************************************************************************
	BCF		STATUS, RP1		; BANK 1
	BSF		TRISA,2			; Set PORTA,2 as input (KEY_DWN)
	BCF		STATUS, RP0		; Goto BANK 0
	BTFSS	KEY_DWN			; Test if KEY_DWN pressed
	GOTO	KEYDWNISR		; If key pressed, go process it

;/****************************************************************************
;* DESCRIPTION: If we arrive here, no keys are pressed, so clear keys flags.
;/****************************************************************************
	CLRF	KEYS_FLAGS		; If no key pressed, clear Keys pressed flags.
	CLRF	KEYUPCNT		; Clear Key UP press counter
	CLRF	KEYDWNCNT		; Clear Key DOWN press counter
	CLRF	KEYONOFFCNT		; Clear Key ONOFF press counter
	MOVLW	D'3'			; Reset key debounce counter
	MOVWF	KEY_DEBOUNCE		;
	CLRF	KEY_TURBO		; Reset Key turbo counter
	GOTO	EXITTMR0		; Done, exit.

;/****************************************************************************
;* DESCRIPTION: Process key Up
;/****************************************************************************
KEYUPISR
	BTFSC	KEYS_FLAGS,0	; First pass? If no, use debounce/turbo	
	GOTO	CONTDEB			; Continue, if not.
	CLRF	TMR_1MS			; Clear 1 ms counter (for 100 ms)
	BSF		BUZZEREN		; Enable beep (for 100ms)
	GOTO	CONTUPISR2		;

CONTDEB
	MOVF	KEY_TURBO,W		; If KEY_TURBO == 255
	SUBLW	D'255'			; Let it be 255 (we are at maximum TURBO speed).
	BTFSS	STATUS,Z		; Else
	INCF	KEY_TURBO,F		; Increment KEY_TURBO

	DECFSZ	KEY_DEBOUNCE,F	; KEY_DEBOUNCE--;
	GOTO	EXITKEYUPISR	; if (KEY_DEBOUNCE == 0)

	MOVF	KEY_TURBO,W		; if (KEY_TURBO < 20)
	SUBLW	D'19'			;
	BTFSS	STATUS,C		;
	GOTO	CONTUPISR0		;
	MOVLW	0x06			; KEY_DEBOUNCE = 6;
	MOVWF	KEY_DEBOUNCE	;
CONTUPISR0
	MOVF	KEY_TURBO,W		; if ((KEY_TURBO > 20) && (KEY_TURBO <= 40))
	SUBLW	D'20'			;
	BTFSC	STATUS, C		;
	GOTO	CONTUPISR1		;
	MOVF	KEY_TURBO,W		;
	SUBLW	D'40'			;
	BTFSS	STATUS, C		;
	GOTO	CONTUPISR1		;
	MOVLW	0x03			; KEY_DEBOUNCE = 3;
	MOVWF	KEY_DEBOUNCE	;
	GOTO	CONTUPISR2		;
CONTUPISR1
	 MOVF	KEY_TURBO,W		; else if (KEY_TURBO > 40)
	 SUBLW	D'40'			;
	 BTFSC	STATUS, C		;
	 GOTO   CONTUPISR2		;
	 MOVLW  D'255'			; KEY_TURBO = 255; 
	 MOVWF  KEY_TURBO		;
	 MOVLW  0x01			; KEY_DEBOUNCE = 1;
	 MOVWF  KEY_DEBOUNCE	;
CONTUPISR2
	BSF		KEYS_FLAGS,0	;
	BTFSS	MODETEMP		; Are we in temperature mode?
	GOTO	INCTIMERSET		; No, we are in timer mode
	;MOVLW	D'10'			; Add 10
	;ADDWF	TEMPER,F		; To temperature

	MOVF	TEMPER,W		; If TEMPER == 250
	SUBLW	D'250'
	BTFSC	STATUS,Z		;
	GOTO	LLA1			; get out if = 250

	INCF	TEMPER,F
	GOTO	LLA				; continue
INCTIMERSET
	MOVF	TIMER,W			; Is Time = 240?
	SUBLW	D'240'			;
	BTFSC	STATUS,Z		;
	GOTO	LLA1			; Yes
	INCF	TIMER,F			; No, increment timer value
LLA
	CALL	SETDISPVAR		; Convert
	DECFSZ	KEYUPCNT,W		; Is this the first time the key UP is pressed?
	GOTO	EXITKEYUPISR	; Exit
LLA1
	CLRF	TMR_1MS			; Clear 1 ms counter (for 100 ms)
	BSF		BUZZEREN		; Enable beep (for 100ms)
EXITKEYUPISR
	GOTO	EXITTMR0		; Done, exit

;/****************************************************************************
;* DESCRIPTION: Process key Timer
;/****************************************************************************
KEYTIMERISR
	BTFSC	KEYS_FLAGS,3	; First pass? If no, return	
	GOTO	EXITTMR0		; Exit
	BTFSC	MODETIMER		; Don't beep if mode already set
	GOTO	EXITTMR0		; Exit
	BSF		KEYS_FLAGS,3	; Set KEY_UP flag
	BCF		MODETEMP		; Unset mode temperature
	BSF		MODETIMER		; Set mode timer
	CALL	SETDISPVAR		; Convert temperture or timer to bcd
	CLRF	TMR_1MS			; Clear 1 ms counter (for 100 ms)
	BSF		BUZZEREN		; Enable beep (for 100ms)
	GOTO	EXITTMR0		; Done, exit

;/****************************************************************************
;* DESCRIPTION: Process key On/Off
;/****************************************************************************
KEYONOFFISR
	INCF	KEYONOFFCNT,F	; Increment key down pressed counter
	BTFSC	KEYS_FLAGS,4	; Oven ON?	
	GOTO	OVENISON		; Yes, let's see if you want to turn it off...

	BTFSC	OVENON			; Is oven On?
	GOTO	OVENISON		; Yes, let's continue...
	MOVF	KEYONOFFCNT,W	; No, lets test how many kys ticks we have
	SUBLW	D'2'			; Do we have 2 keys ticks?
	BTFSS	STATUS, Z		;
	GOTO	CONTONOFF		; No, keep counting...
SETOVENON
	BSF		OVENON			; No turn oven on
	BSF		LEDBLINK		; Enable Led ONOFF blinking
	BSF		MODETEMP		; Set temperature mode
	BCF		MODETIMER		; Clear timer mode
	BSF		KEYS_FLAGS,4	; Set OVEN On flag
	GOTO	EXITTMR0		; Done, exit.
OVENISON
	BTFSC	MODEPAUSE
	GOTO	CLRPAUSEMODE
	BSF		MODEPAUSE
	GOTO	ENDPAUSETEST
CLRPAUSEMODE
	BCF		MODEPAUSE
ENDPAUSETEST
	MOVF	KEYONOFFCNT,W	; Test if we have
	SUBLW	D'6'			; 6 keys ticks..
	BTFSS	STATUS, Z		; If yes...
	GOTO	CONTONOFF		; No, we have not... Beep and continue
	BCF		OVENON			; Yes, turn oven off
	BCF		LEDBLINK		; Disable Led ONOFF blink
	GOTO	EXITTMR0		; Done, exit


	MOVF	KEYONOFFCNT,W	; Are key onoff ticks >= 8?
	SUBLW	D'7'			;
	BTFSC	STATUS, C		;
	GOTO	CONTONOFF		; No...
	MOVLW	D'8'			; Yes, let it be...
	MOVWF	KEYONOFFCNT		; 8...
	GOTO	EXITTMR0		; Done...

CONTONOFF
	CLRF	TMR_1MS			; Clear 1 ms counter (for 100 ms)
	BSF		BUZZEREN		; Enable beep (for 100ms)
	GOTO	EXITTMR0

;/****************************************************************************
;* DESCRIPTION: Process key Light
;/****************************************************************************
KEYLIGHTISR
	BTFSC	KEYS_FLAGS,5	; first pass? If no, return	
	GOTO	EXITTMR0
	BSF		KEYS_FLAGS,5	; set KEY_UP flag
	BTFSS	MODELIGHT
	GOTO	SETLIGHTON
	GOTO	SETLIGHTOFF
SETLIGHTON
	BSF		MODELIGHT
	GOTO	LIGHTCONT
SETLIGHTOFF
	BCF		MODELIGHT
LIGHTCONT
	CLRF	TMR_1MS			; Clear 1 ms counter (for 100 ms)
	BSF		BUZZEREN		; Enable beep (for 100ms)
	GOTO	EXITTMR0

;/****************************************************************************
;* DESCRIPTION: Process key Toaster
;/****************************************************************************
KEYTOASTERISR
	BTFSC	KEYS_FLAGS,6	; first pass? If no, return	
	GOTO	EXITTMR0		; No, return
	BSF		KEYS_FLAGS,6	; Set KEY_UP flag
	BTFSS	MODETOASTER		; Are we in toaster mode (toaster on)?
	GOTO	SETTOASTERON	; No, set it
	GOTO	SETTOASTEROFF	; Yes, clear it
SETTOASTERON
	BSF		MODETOASTER		; Set toaster mode
	GOTO	TOASTERCONT		; End
SETTOASTEROFF
	BCF		MODETOASTER		; Clear toaster mode
TOASTERCONT
	CLRF	TMR_1MS			; Clear 1 ms counter (for 100 ms)
	BSF		BUZZEREN		; Enable beep (for 100ms)
	GOTO	EXITTMR0		; Done

;/****************************************************************************
;* DESCRIPTION: Process key Temperature
;/****************************************************************************
KEYTEMPISR
	BTFSC	KEYS_FLAGS,7	; First pass? If no, return	
	GOTO	EXITTMR0		; No, return
	BTFSC	MODETEMP		; Don't beep if mode already set
	GOTO	EXITTMR0		; If already in temperature mode, exit
	BSF		KEYS_FLAGS,7	; set KEY_UP flag
	BSF		MODETEMP		; Set mode to temperature (if not set)
	BCF		MODETIMER		; Clear timer mode (if set)
	CALL	SETDISPVAR		; Convert temperture or timer to bcd
	CLRF	TMR_1MS
	BSF		BUZZEREN		; Enable beep
	GOTO	EXITTMR0

;/****************************************************************************
;* DESCRIPTION: Process key Down
;/****************************************************************************
KEYDWNISR
	BTFSC	KEYS_FLAGS,1	; First pass? If no, use debounce/turbo	
	GOTO	CONTDEB1		; Continue, if not.
	CLRF	TMR_1MS			; Clear 1 ms counter (for 100 ms)
	BSF		BUZZEREN		; Enable beep (for 100ms)
	GOTO	CONTDWNISR2		;

CONTDEB1
	MOVF	KEY_TURBO,W		; If KEY_TURBO == 255
	SUBLW	D'255'			; Let it be 255 (we are at maximum TURBO speed).
	BTFSS	STATUS,Z		; Else
	INCF	KEY_TURBO,F		; Increment KEY_TURBO

	DECFSZ	KEY_DEBOUNCE,F	; KEY_DEBOUNCE--;
	GOTO	EXITKEYDWNISR	; if (KEY_DEBOUNCE == 0)

	MOVF	KEY_TURBO,W		; if (KEY_TURBO < 20)
	SUBLW	D'19'			;
	BTFSS	STATUS,C		;
	GOTO	CONTDWNISR0		;
	MOVLW	0x06			; KEY_DEBOUNCE = 6;
	MOVWF	KEY_DEBOUNCE	;
CONTDWNISR0
	MOVF	KEY_TURBO,W		; if ((KEY_TURBO > 20) && (KEY_TURBO <= 40))
	SUBLW	D'20'			;
	BTFSC	STATUS, C		;
	GOTO	CONTDWNISR1		;
	MOVF	KEY_TURBO,W		;
	SUBLW	D'40'			;
	BTFSS	STATUS, C		;
	GOTO	CONTDWNISR1		;
	MOVLW	0x03			; KEY_DEBOUNCE = 3;
	MOVWF	KEY_DEBOUNCE	;
	GOTO	CONTDWNISR2		;
CONTDWNISR1
	MOVF	KEY_TURBO,W		; else if (KEY_TURBO > 40)
	SUBLW	D'40'			;
	BTFSC	STATUS, C		;
	GOTO	CONTDWNISR2		;
	MOVLW	D'255'			; KEY_TURBO = 255; 
	MOVWF	KEY_TURBO		;
	MOVLW	0x01			; KEY_DEBOUNCE = 1;
	MOVWF	KEY_DEBOUNCE	;
CONTDWNISR2
	BSF		KEYS_FLAGS,1	;
	BTFSS	MODETEMP		; Are we in temperature mode?
	GOTO	DECTIMERSET		; No, we are in timer mode

	MOVF	TEMPER,W		; If TEMPER == 40
	SUBLW	D'40'
	BTFSC	STATUS,Z		;
	GOTO	LLB1			; get out if = 40
	;MOVLW	D'10'			; Else subtract 10
	;SUBWF	TEMPER,F		; From temperature
	DECF	TEMPER,F
	GOTO	LLB				; continue
DECTIMERSET
	MOVF	TIMER,W			; Is Time = 0?
	BTFSC	STATUS,Z		;
	GOTO	LLB1			; Yes
	DECF	TIMER,F			; No, decrement timer value
LLB
	CALL	SETDISPVAR		; Convert
	DECFSZ	KEYDWNCNT,W		; Is this the first time the key DOWN is pressed?
	GOTO	EXITKEYDWNISR	; Exit
LLB1
	CLRF	TMR_1MS			; Clear 1 ms counter (for 100 ms)
	BSF		BUZZEREN		; Enable beep (for 100ms)
EXITKEYDWNISR
	GOTO	EXITTMR0		; Done, exit

;/****************************************************************************
;* DESCRIPTION: End of key process, restore tris and re-enable ints...
;/****************************************************************************
EXITTMR0
	BSF		STATUS, RP0		; Goto
	BCF		STATUS, RP1		; BANK 1
	BCF		TRISB,7			; Set PORTB,7 as output (KEY_UP)
	BCF		TRISB,6			; Set PORTB,6 as output (KEY_TIMER)
	BCF		TRISB,5			; Set PORTB,5 as output (KEY_ONOFF)
	BCF		TRISB,4			; Set PORTB,4 as output (KEY_LIGHT)
	BCF		TRISA,5			; Set PORTA,5 as output (KEY_TOASTER)
	BCF		TRISA,4			; Set PORTA,4 as output (KEY_TEMP)
	BCF		TRISA,2			; Set PORTA,2 as output (KEY_DWN)
	BCF		STATUS, RP0		; Goto
	BCF		STATUS, RP1		; BANK 0

ENDTMR0
	GOTO	END_INT			; Get out.

;****************************************************************************
;* For Timer 1 Interrupt calculation:
;* Overflow: 1/((FHz/4)/T1_Divider) = 1/(8000000/1) = 1/8000000 = 0,0000005s=0.5us.
;* For an overflow of 1ms:
;* 0,001 = 0,000001 * X
;* X = 0,001/0,000001 => X = 1000
;* For an overflow of 1ms, preload Timer1 with: 65536-1000, wich is: 64536.
;* Thus:
;* f(interrupt)=1/(freq_cpu/4/prescaler/preloadvalue)
;* As I want one interrupt at every 1 ms:
;* f(interrupt)=1/(4000000/4/1/1000)
;* f(interrupt)= 1/1000 times/second => T(f) = 1/f = 0,001 (1 ms).
;*
;*****************************************************************************
INT_TMR1
	;/****************************************************************************
	;* DESCRIPTION: Clear Timer1 interrupt and reload it's values
	;/****************************************************************************
	BCF		PIR1,TMR1IF		; Clear Timer1 Interrupt flag
	MOVLW	0x0C			;
	MOVWF	TMR1L			;
	MOVLW	0xFE			;
	MOVWF	TMR1H			; Init TIMER1 with decimal 65036 (interrupt every 500us)

	;/****************************************************************************
	;* DESCRIPTION: Turn off al displays and leds
	;/****************************************************************************
	BSF		LED_CMN			; Leds OFF
	BSF		DISP3			; Display3 OFF
	BSF		DISP2			; Display2 OFF
	BSF		DISP1			; Display1 OFF

	;/****************************************************************************
	;* DESCRIPTION: Clear PORTA and PORTB
	;/****************************************************************************
	MOVLW	B'11111111'
	MOVWF	PORTA
	MOVWF	PORTB

	;/****************************************************************************
	;* DESCRIPTION: Set INDEX_DISP to  value between 0 and 3
	;/****************************************************************************
	INCF	INDEX_DISP,F	; Increment display index
	MOVLW	D'3'			; Limit counter from 0 to 3
	ANDWF	INDEX_DISP,F	;	"		"		"

	;/****************************************************************************
	;* DESCRIPTION: Test to see if the oven is ON
	;/****************************************************************************
	BTFSS	OVENON
	GOTO	END_INT

	;/****************************************************************************
	;* DESCRIPTION: See what we need to process now, depending on INDEX_DISP
	;/****************************************************************************
	MOVLW	D'0'
	XORWF	INDEX_DISP,W	; If index = 0
	BTFSC	STATUS,Z		; Then it's time
	GOTO	SETLEDS			; to show LEDS
	MOVLW	D'1'
	XORWF	INDEX_DISP,W	; If index = 1
	BTFSC	STATUS,Z		; Then it's time
	GOTO	SETDISP3		; to show DISPLAY1
	MOVLW	D'2'
	XORWF	INDEX_DISP,W	; If index = 2
	BTFSC	STATUS,Z		; Then it's time
	GOTO	SETDISP2		; to show DISPLAY2
	MOVLW	D'3'
	XORWF	INDEX_DISP,W	; If index = 3
	BTFSC	STATUS,Z		; Then it's time
	GOTO	SETDISP1		; to show DISPLAY3

SETLEDS
	;/****************************************************************************
	;* DESCRIPTION: Process all leds
	;/****************************************************************************
    BCF		LED_UP			; Led UP always on
    BCF		LED_DWN			; Led DWN always on
    BTFSC	MODELIGHT		; Is led light on?
    GOTO	$+3
    BSF		LED_LIGHT		; No
    GOTO	$+2
    BCF		LED_LIGHT		; Yes
    BTFSC	MODETOASTER		; Is led toast on?
    GOTO	$+3
    BSF		LED_TOASTER		; No
    GOTO	$+2
    BCF		LED_TOASTER		; Yes
    BTFSC	MODETIMER		; Is mode time on?
    GOTO	$+3
    BSF		LED_TIMER		; No
    GOTO	$+2
    BCF		LED_TIMER		; Yes
    BTFSC	MODETEMP		; Is mode temp on?
    GOTO    $+3
    BSF		LED_TEMP		; No
    GOTO	$+2
    BCF		LED_TEMP		; YES
	BCF		LED_CMN			; Leds ON
	GOTO	$+1				; 2 us delay (transistor response time)
	GOTO	CONT_INT

SETDISP3
	;/****************************************************************************
	;* DESCRIPTION: Process Display3 - HUNDREDS
	;/****************************************************************************
	BTFSC	MODEPRE			; Pre heating mode?
	GOTO	MODEPRECNT		; Yes
	MOVF	HUNDREDS,W		; No, test to see if HUNDREDS
	XORLW	D'0'			; are zero (00) so
	BTFSC	STATUS,Z		; we turn display off
	GOTO	CONT_INT
	MOVF	HUNDREDS,W		; copy again hundeds to W (was cleared?)
	CALL	DECDISP_A		; convert
	MOVWF	PORTA			; put it in PORTA
	MOVF	HUNDREDS,W		; copy hundeds to W
	CALL	DECDISP_B		; convert
	MOVWF	PORTB			; put it in PORTB
	GOTO	CONTHUNS
MODEPRECNT
	MOVLW	D'10'			; Load 'P' letter from table
	CALL	DECDISP_A		; Convert
	MOVWF	PORTA			; Put in Porta
	MOVLW	D'10'			; Load 'P' letter from table
	CALL	DECDISP_B		; Convert
	MOVWF	PORTB			; Put in Portb
CONTHUNS
	BCF		DISP3			; display 3 on
	GOTO	$+1				; 2 us delay (transistor response time)
	GOTO	CONT_INT

SETDISP2
	;/****************************************************************************
	;* DESCRIPTION: Process Display2 - TENS
	;/****************************************************************************
	BTFSC	MODEPRE			; Mode pre heating?
	GOTO	MODEPRECNT2		; Yes
	MOVF	HUNDREDS,W		; Test to see if HUNDREDS
	XORLW	D'0'			; are zero (00) so
	BTFSS	STATUS,Z		; if TENS are also 0, we turn it off.
	GOTO	CONTTENS
	MOVF	TENS,W			; If we get here, HUNDREDS are zero, so test to see if TENS also 0
	XORLW	D'0'			; Are TENS = 0?
	BTFSC	STATUS,Z		; Test it
	GOTO	CONT_INT		; Yes, let both HUNDREDS and TENS digits off!
CONTTENS
	MOVF	TENS,W			; copy tens to W
	CALL	DECDISP_A		; convert
	MOVWF	PORTA			; put it in PORTA
	MOVF	TENS,W			; copy tens to W
	CALL	DECDISP_B		; convert
	MOVWF	PORTB			; put it in PORTB
	GOTO	CONTTENS2
MODEPRECNT2
	MOVLW	D'11'			; Load 'r' letter from table
	CALL	DECDISP_A		; Convert
	MOVWF	PORTA			; Put in Porta
	MOVLW	D'11'			; Load 'r' letter from table
	CALL	DECDISP_B		; Convert
	MOVWF	PORTB			; Put in Portb
CONTTENS2
	BCF		DISP2			; display 2 on
	GOTO	$+1				; 2 us delay (transistor response time)
	GOTO	CONT_INT

SETDISP1
	;/****************************************************************************
	;* DESCRIPTION: Process Display1 - UNITS
	;/****************************************************************************
	BTFSC	MODEPRE			; Mode pre heating?
	GOTO	MODEPRECNT3		; Yes
	MOVF	UNITS,W			; copy UNITS to W
	CALL	DECDISP_A		; convert
	MOVWF	PORTA			; put it in PORTA
	MOVF	UNITS,W			; copy UNITS to W
	CALL	DECDISP_B		; convert
	MOVWF	PORTB			; put it in PORTB
	GOTO	CONTUNS2
MODEPRECNT3
	MOVLW	D'12'			; Load 'e' letter from table
	CALL	DECDISP_A		; Convert
	MOVWF	PORTA			; Put in Porta
	MOVLW	D'12'			; Load 'e' letter from table
	CALL	DECDISP_B		; Convert
	MOVWF	PORTB			; Put in Portb
CONTUNS2
	BCF		DISP1			; display 1 on
	GOTO	$+1				; 2 us delay (transistor response time)

CONT_INT
	;/****************************************************************************
	;* DESCRIPTION: If beep enabled, toggle buzzer pin.
	;/****************************************************************************
	BTFSS	BUZZEREN		; Buzzer Enabled?
	GOTO	ENDBEEPC		; No, get out
	BSF		STATUS, RP0		; Bank 1
	BCF		TRISC,0			; PORTC pin0 output
	MOVLW	D'1'			; Pin 0 => Buzzer
	BCF		STATUS, RP0		; Bank 0
	XORWF	PORTC,F			; Xor PORTC,pin0 with decimal 1 (Binary 00000001).
ENDBEEPC
	;/****************************************************************************
	;* DESCRIPTION: Initialize all interrupts values for TIMER0 and TIMER1
	;/****************************************************************************
	INCF	TMR_1MS,F		; Increment TR_1MS (0.5 ms counter)
	MOVLW	D'200'
	XORWF	TMR_1MS,W		; Is TMR_1MS = 200 (100ms)?
	BTFSC	STATUS,Z		; No, skip if TMR_1MS < 200
	GOTO	INC100MS		; Yes, zero TMR_1MS and inc TMR_100MS
	GOTO	END_INT			; No, exit interrupt

INC100MS
	;/****************************************************************************
	;* DESCRIPTION: Disable buzzer at every 100ms and process 100ms counter
	;/****************************************************************************
	BCF		BUZZEREN		; Disable buzzer
	CLRF	TMR_1MS			; Zero TMR_1MS
	INCF	TMR_100MS,F		; Increment TMR_100MS (100 ms counter)
	MOVLW	D'5'			; 500 ms?
	XORWF	TMR_100MS,W		; Is TMR_100MS = 5 (500ms)?
	BTFSC	STATUS,Z		; No, skip if TMR_100MS < 5
	GOTO	INCCONTA		; Yes, zero TMR_100MS and inc 0,5 seconds counter
	GOTO	END_INT			; No, Exit interrupt

INCCONTA
	;/****************************************************************************
	;* DESCRIPTION: Process 100 ms counter. Blink led if enabled.
	;/****************************************************************************
	CLRF	TMR_100MS		; Zero 100 ms counter (reached 500ms)
	MOVLW	D'1'			; Verify if ...
	XORWF	T1_CONTA,W		; passed 1/2 second ...
	BTFSS	STATUS,Z		;
	GOTO	LEDOUT			; No.
							; yes.
	BSF		STATUS, RP0		; Bank 1
	BCF		TRISC,6			; PORTC pin0 output (LED_ONOFF)
	BCF		STATUS, RP0		; Bank 0
	BTFSS	MODEPAUSE		; Are we in pause mode?
	GOTO	NOPAUSE
	MOVLW	B'01000000'		; Pin 6 => LED_ONOFF
	XORWF	PORTC,F			; Xor PORTC,pin0 with decimal 64 (Binary 01000000).
	GOTO	LEDOUT
NOPAUSE
	BSF		LED_ONOFF		; Not in pause, let led on/off on all time
LEDOUT
	DECFSZ	T1_CONTA,F		; Decrement the Counter Register
	GOTO	END_INT			; If diferent of zero, exit interrupt
	MOVLW	1				;
	MOVWF	T1_CONTA		; Restart Counter Register with 2

END_INT
	;/****************************************************************************
	;* DESCRIPTION: Restore all the context saved at start of interrupt isr
	;/****************************************************************************
	MOVF	PCLATH_TEMP,w	; retrieve copy of PCLATH register
	MOVWF	PCLATH			; restore pre-isr PCLATH register contents
	MOVF	FSR_TEMP,W		; Put FSR_TEMP value in W
	MOVWF	FSR				; Recover register FSR (Indirect Addressing)
	MOVF    STATUS_TEMP,w	; retrieve copy of STATUS register
	MOVWF	STATUS			; restore pre-isr STATUS register contents
	SWAPF   W_TEMP,f
	SWAPF   W_TEMP,w		; restore pre-isr W register contents

	;/****************************************************************************
	;* DESCRIPTION: Done interrupts, return.
	;/****************************************************************************
	RETFIE					; Return from interrupt

;/****************************************************************************
;* DESCRIPTION: binary_to_bcd - 8-bits
;* INPUT:		TEMPER   - 8-bit binary number
;* RETURN:      HUNDREDS - the HUNDREDS digit of the BCD conversion
;*				TENS     - the TENS digits of the BCD conversion
;*				UNITS    - the ones digits of the BCD conversion
;*****************************************************************************/ 
BIN2BCD
	MOVWF	AUX				; save the value to convert in aux
	CLRF	UNITS			;
	CLRF	TENS			;
	CLRF	HUNDREDS		; RESET variables
	MOVF	AUX,F			;
	BTFSC	STATUS,Z		; Is the value to convert = 0?
	RETURN					; Yes - Return
							; No
	INCF	UNITS,F			; Increment unit
	MOVF	UNITS,W			;
	XORLW	0X0A			;
	BTFSS	STATUS,Z		; unit = 10d ?
	GOTO	$+3				; No
							; yes
	CLRF	UNITS			; Reset unit
	INCF	TENS,F			; Increment TENS
	MOVF	TENS,W			;
	XORLW	0X0A			;
	BTFSS	STATUS,Z		; Tens = 10d ?
	GOTO	$+3				; No
							; Yes
	CLRF	TENS			; Reset TENS
	INCF	HUNDREDS,F		; Increment HUNDREDS
	DECFSZ	AUX,F			; End of convertion ?
	GOTO	$-.14			; No - go back to continue the convertion
	RETURN					; Yes

;/****************************************************************************
;* DESCRIPTION: Set bcd values for temperature or time
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 
SETDISPVAR
	BTFSS	MODETEMP		; Are we in temperature mode?
	GOTO	TIMERMODE		; No, we are in timer mode
	MOVF	TEMPER,W		; Put temperature in W
	GOTO	LL2				; continue
TIMERMODE
	MOVF	TIMER,W			; Put time in W
LL2
	CALL	BIN2BCD			; Decode time to bcd
	RETURN

;/****************************************************************************
;* DESCRIPTION: ms delay
;* return:      none
;* ALGORITHM:   Call with the number of ms in W
;* NOTES:       none
;*****************************************************************************/ 
;; Timing analysis (at 4 MHz clock)
;; Load and call       =  3                     =    3
;; Static overhead = A =  9 + 4 * 247           =  997
;; Outer loop      = B = 12 * (delay-1)         = - 12 +  12 * delay
;; Inner loop      = C =  4 * 247 * (delay-1)   = -988 + 988 * delay
;; Total           = 1000 * delay

DELAY_MS
        MOVLW   D'248'			;
        MOVWF   DELAY_TEMP		;
DELAY_MS_LOOP
        ADDLW   -1              ;
        BTFSS   STATUS,Z        ;
        GOTO    DELAY_MS_LOOP   ;
        DECF    DELAY_TEMP,F    ;
        BTFSC   STATUS,Z        ;
        RETURN                  ;
        GOTO    $+1             ;
        NOP                     ;
        MOVLW   D'248'			;
        GOTO    DELAY_MS_LOOP   ;


;/****************************************************************************
;* DESCRIPTION: Program Initialization
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 
SET_UP

	;/****************************************************************************
	;* DESCRIPTION: First, disable everything we don't need
	;/****************************************************************************
	BCF		STATUS,RP0		; RP0 = 0 -> Bank 0
	BCF		SSPCON,SSPEN	; Disable Synchronous Serial Port
	BCF		STATUS,RP0		; RP0 = 0
	BSF		STATUS,RP1		; RP1 = 1 -> Bank 2 (I need it to switch to bank2)
	CLRF	ANSEL			; digital I/O
	CLRF	ANSELH			;	"	"	"
	MOVLW	B'00000000'		; To disable comparators
	MOVWF	CM1CON0			; disable comparator
	MOVLW	B'00000000'		; Not necessary?
	MOVWF	CM2CON0			; disable 2nd comparator
	MOVLW	B'00000000'		; Not necessary?
	MOVWF	CM2CON1			; disable 3rd comparator

	;/****************************************************************************
	;* DESCRIPTION: Now, set TRIS directions for PORTA, PORTB and PORTC
	;/****************************************************************************
	BSF		STATUS,RP0		; RP0 = 1 -> Bank 1
	BCF		STATUS,RP1		; RP1 = 0 -> For Bank 1 ( need it to came back from bank2)
	MOVLW	B'11001011'		; RA0,RA1,RA2,RA4,RA5,RA6 and RA7 output - RA3 input
	MOVWF	TRISA			; Set it
	MOVLW	B'00001111'		; All PORTB output
	MOVWF	TRISB			; Set it
	MOVLW	B'10000010'		; RC0,RC2,RC3,RC4,RC5,RC6 output - RC1 and RC7 inputs
	MOVWF	TRISC			; Set it

	;/****************************************************************************
	;* DESCRIPTION: Disable all leds & displays, Set temperature and timer values
	;/****************************************************************************
	BCF		STATUS,RP0		; RP0 = 1 -> Bank 1
	BSF		LED_CMN			; Leds off
	BSF		DISP1			; Display1 off
	BSF		DISP2			; Display2 off
	BSF		DISP3			; Display3 off
	BCF		LED_ONOFF		; Led ONOFF off
	MOVLW	D'180'			; Init temperature with 180 degrees Celsius
	MOVWF	TEMPER			; Put it in TEMPER file
	MOVLW	D'40'			; Start TIMER with 40 minutes
	MOVWF	TIMER			; Put it in TIMER file

	;/****************************************************************************
	;* DESCRIPTION: Init buzzer duration and clear units, tens ad hundreds files
	;/****************************************************************************
	MOVLW	D'200'
	MOVWF	BUZ_LENGTH		; Buzzer initial duration
	CLRF	UNITS			; Clear dislay's registers
	CLRF	TENS			;	"	"	"	"	"
	CLRF	HUNDREDS		;	"	"	"	"	"

	;/****************************************************************************
	;* DESCRIPTION: Set start modes and convert starting temperature value
	;/****************************************************************************
	BCF		MODEPRE			; Put pre heating mode off
	BSF		MODETEMP		; Temperture mode on
	BCF		MODETIMER		; Timer mode off
	BCF		MODELIGHT		; Internal light off
	BCF		MODETOASTER		; Toaster off
	BSF		MODEPAUSE		; Start oven in pause mode
	BSF		LEDBLINK		; Led ON/OFF Blinking enabled.
	BCF		OVENON			; Start with Oven OFF
	CALL	SETDISPVAR		; Convert temperture or timer to bcd

	;/****************************************************************************
	;* DESCRIPTION: Clear Ports A and B (just in case).
	;/****************************************************************************
	MOVLW	B'11111111'
	MOVWF	PORTA
	MOVLW	B'11111111'
	MOVWF	PORTB
	BSF		STATUS,RP0		; RP0 = 1 -> Bank 1

	;/****************************************************************************
	;* DESCRIPTION: Initialize all interrupts values for TIMER0 and TIMER1
	;/****************************************************************************
	MOVLW	B'00000111'
	MOVWF	OPTION_REG
	BCF		STATUS,RP0		; RP0 = 0 -> Bank 0
	MOVLW	B'00000000'		; Configure TIMER1 for internal clock and prescaler 1:1
	MOVWF	T1CON			; Init Timer1
	MOVLW	0x0C			;
	MOVWF	TMR1L			;
	MOVLW	0xFE			;
	MOVWF	TMR1H			; Init TIMER1 with decimal 65036 (interrupt every 500us (2KHz))
	BSF		STATUS,RP0		; Bank 1
	MOVLW   B'00000001'		; Enable TIMER1 Overflow Interrupt
	MOVWF	PIE1
	BCF		STATUS,RP0		; Bank 0
	CLRF	PIR1			; Clear Peripheral Interrupt Flags
	BSF		T1CON, TMR1ON	; Start TMR1
	MOVLW	TMR0VAL			; Interrupt Timer 0 every 100ms (at 4MHz)
	MOVWF	TMR0			;

	;/****************************************************************************
	;* DESCRIPTION: Enable and start all interrupts
	;/****************************************************************************
	BSF		INTCON, T0IE	; Enable Timer0 interrupt.
	BSF		INTCON, PEIE	; Enable Peripheral Interrupts
	BCF		INTCON, T0IF	; Timer 0 Overflow Interrupt Flag bit = reset at every Timer0 int
	BSF		INTCON, GIE		; Enable Global Interrupts

;/****************************************************************************
;* DESCRIPTION: Main Program
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 
MAIN

;/****************************************************************************
;* DESCRIPTION: Disable interrupts, so we don't interfere in displays
;/****************************************************************************
	;BCF		INTCON, GIE		; Disable Global Interrupts

;/****************************************************************************
;* DESCRIPTION: Re-enable interrupts, so we can continue processing displays
;/****************************************************************************
	;BSF		INTCON, GIE		; Enable Global Interrupts

;/****************************************************************************
;* DESCRIPTION: Give some time so the displays are not dimmed.
;/****************************************************************************
	MOVLW	D'100'
	CALL	DELAY_MS

;/****************************************************************************
;* DESCRIPTION: Restart all again...
;/****************************************************************************
	GOTO	MAIN

	END